/*

	Let's try this out with a scored version of Rock, Paper, Scissors.
	The first player to get to 3 wins.

	We'll be using the default skin and many of the default datum types.

	(Implementation-specific details used for the demo that have little to do
	with the library itself are hidden in magic.dm.)

*/

////////////////////////////////////////////////////////////////////////////////
/*
	We need a room handler somewhere.
*/
//	I'm going to make one global.  You don't have to.
var/bgf_RoomHandler/Demo/gRooms	= new

////////////////////////////////////////////////////////////////////////////////
/*
	The library's verbs will need a way to access the handler of the mob that
	calls them.  Even if you don't use the verbs provided, you may still want
	to define this so you can interpret mouse clicks.
*/
mob/Demo
	bgf_GetRoomHandler()
		return gRooms

////////////////////////////////////////////////////////////////////////////////
/*
	We also need to track connections.
*/
mob/Demo
	Login()
		..()
		var/bgf_RoomHandler/H	= src.bgf_GetRoomHandler()
		if ( istype(H) )		H.p_Reconnected(src)
	Logout()
		var/bgf_RoomHandler/H	= src.bgf_GetRoomHandler()
		if ( istype(H) )		H.p_Disconnected(src)
		..()

////////////////////////////////////////////////////////////////////////////////
/*
	The default interface provides a way to set a default type of room.
	If you don't define this, you'll end up with a basic bgf_Room.
*/
bgf_RoomHandler/Demo
	bgf_GetNewRoomType(Person)
		return /bgf_Room/Demo

////////////////////////////////////////////////////////////////////////////////
/*
	Unless creating something really abstract, you'll likey want to use a map.
	You could try sharing one space, but it might be easier to divide the map
	between the rooms.
*/
bgf_Room/Demo
	//	I'm claiming a section when the room is created...
	New(bgf_RoomHandler/Handler)
		..(Handler)
		src.ClaimMap()
	//	...and freeing it up when the room is destroyed.
	Del()
		src.ReturnMap()
		..()

/*
	You'll want to make sure that people entering the room are placed in
	the proper section of the map.
*/
bgf_Room/Demo
	p_Entered(Person)
		..(Person)
		if ( istype(Person,/atom/movable) )	src.PlaceOnEnter(Person)

/*
	You'll also want to make sure that those people are viewing the section of
	the map you desire.
*/
bgf_Room/Demo
	p_AddInterfaceRoom(Person)
		.			= ..(Person)
		if ( !(.) )	return .

		var/client/C				= Person
		if ( ismob(Person) )
			var/mob/M				= Person
			C						= M.client
		if ( !istype(C,/client) )	return FALSE

		//	I'm going to set the view to a 3x3 section completely in the map.
		//	Normally, I'd set something this static in the client itself.
		//	However, it's probably good to get in the habit in case the settings
		//	are more dynamic.  For instance, what if you have more than one type
		//	of room available in the same project?
		C.perspective	= EDGE_PERSPECTIVE
		C.view			= "3x3"

		src.AddMagic(C)

		//	BTW, remember to include help files for your projects!  This can be
		//	especially important for board games as they have a lot of variety.
		//	A browser is included in the prototype interface if you wish to use it.
		//
		//	That said, the button to access the browser uses .winset and I've
		//	experienced problems with Vista not displaying the browser a second time.
		//	(Unless forced to redraw with a child splitter or window size adjustment.)
		//	I tend to use a verb that outputs the files again instead.
		Person	<< browse('bgf_demo.css',"display=0")
		Person	<< output('bgf_demo.html',"help")

/*
	The map should be initialized at some point.  I'm choosing to do it when the
	game is started so the results of the last game will stay on the screen between
	matches.  You might want to do this in bgf_Room.p_Ended() instead.

	Naturally, assigning pieces and players is also good.
*/
bgf_Room/Demo
	p_Started(Starter,AutomatedCall=FALSE)
		src.ResetMap()
		src.AssignHands()
		return ..(Starter,AutomatedCall)
////////////////////////////////////////////////////////////////////////////////
/*

	Define the maximum number of players (if any).  By default, this will also
	start games automatically whenever there are enough people waiting.

*/
bgf_Room/Demo
	p_GetMaxPlayers()
		return 2

////////////////////////////////////////////////////////////////////////////////
/*
	Now for the meat.  You have to define the phases of your game.

	I'm going to create my own type of phase handler.  The room should know what
	it is.
*/
bgf_Room/Demo
	bgf_GetPhaseHandlerType()
		return /bgf_PhaseHandler/Demo

/*
	Meanwhile, the phase handler should know what its first phase is.
*/
bgf_PhaseHandler/Demo
	bgf_GetStartingPhaseType()
		return /bgf_Phase/Demo
/*
	For later phases, it will ask the phase itself.  Games can change on the
	fly.  I'm going to use a single phase in this game.  You don't have to.
*/
bgf_Phase/Demo
	bgf_GetNextPhaseType()
		if (src.GetWinner())	return null
		return /bgf_Phase/Demo.
/*
	Note that returning null for the next phase type will simply stall the game.
	It won't end it.  For that, you may want to use the phases' ph_EndBody().
*/
bgf_Phase/Demo
	ph_EndBody()
		src.DisplayResults()
		var/winner	= src.GetWinner()
		if (winner)
			spawn(0)
				var/bgf_Room/R		= src.bgf_GetRoom()
				if ( istype(R) )	R.p_End(winner)

		return ..()

/*
	There's also a very important difference from the parent Phase library.  That
	library's default is to only adjust the interfaces of players entered into
	it.  However, bgf_Phases adjust the interfaces of everyone in the room.

	This demo doesn't do a lot of additional interface manipulation (outside of
	the hand images), but be aware when creating your own projects.
*/

////////////////////////////////////////////////////////////////////////////////
/*
	Phases can handle their own input.  We could  define some skin controls with
	the interface procs, but this game is fine with mouse control.

	Phases handle their own clicks (and double clicks).  This allows you to
	localize how the input is interpreted rather than defining a bunch of traits
	in the objects themselves.
*/
bgf_Phase/Demo
	ph_Click(Player,atom/Clicked,Location,Control,Params)
		if ( !src.ph_IsPlaying(Player) )	return FALSE
		return src.InterpretClick(Player,Clicked,Location,Control,Params)
/*
	Of course, that won't do you any good if BYOND's default click behavior
	isn't changed to look for the phase.
*/
atom
	Click(location,control,params)
		if (!usr)	return ..(location,control,params)
		var/bgf_RoomHandler/H	= usr.bgf_GetRoomHandler()
		if ( !istype(H) )		return ..(location,control,params)
		var/bgf_Room/R			= H.p_Find(usr)
		if ( !istype(R) || !R.bgf_phases)	return ..(location,control,params)
		var/bgf_Phase/P			= R.bgf_phases.ph_GetCurrentPhase()
		if ( !istype(P) )		return ..(location,control,params)
		return P.ph_Click(usr,src,location,control,params)

////////////////////////////////////////////////////////////////////////////////
/*
	When games end, rooms can be sent whatever ended them.  Note that the room
	itself will be sent if the room is being deleted.
*/
bgf_Room/Demo
	p_Ended(Ender,AutomatedCall=FALSE)
		if (Ender && (Ender != src))	src.AnnounceWinner(Ender)
		return ..(Ender,AutomatedCall)

////////////////////////////////////////////////////////////////////////////////

/*
	The following are merely suggestions.
*/

//	Name your rooms.
bgf_Room/Demo
	bgf_GetName()
		return "Room #[src.z]"

//	Create default avatars.
bgf_RoomHandler/Demo
	bgf_GetAvatar(Person,Wait=FALSE, Class="bgf_avatar")
		.			= ..(Person,Wait,Class)
		if ( !(.) )	.	= "<img class='[Class]' src='\ref['avatar.dmi']'>"

//	Create room pictures.
bgf_Room/Demo
	bgf_GetPicture()
		return src.bgf_GetAvatar()

//	Welcome your players.
bgf_Room/Demo
	p_Entered(Person,p_Room/OldRoom,AutomatedCall=FALSE)
		Person	<< output("<span class='bgf_admin'>[src.bgf_GetAvatar()] Welcome to [src.bgf_GetName()], [Person].</span>","bgf_chat")
		return ..(Person,OldRoom,AutomatedCall)


//	Feel free to change the MOTD.  Procs have been included to add developer notes.
bgf_MOTD/Demo
	bgf_GetAdminPrefix(Viewer)
		.	= "This is a demo for ACWraith's Board Game Framework."
		.	+= "  It uses the prototype interface included."
		.	+= "  Feel free to hang out in a room or even create another."
		.	+= "  Both creation and movement can be handled through the List Rooms button."
		.	+= "\n\nHosts, please note that the default host check does not recognize guest keys"
		.	+= " so you will have to log into the pager in order to manage a server.\n"

bgf_RoomHandler/Demo
	bgf_GetMOTDType()
		return /bgf_MOTD/Demo
