/*
////////////////////////////////////////////////////////////////////////////////
bgf_MOTD
	parent_type		= /motd_MOTD (MOTD library)

	The room handler is kept track of.
		var/bgf_RoomHandler/bgf_handler
		New(bgf_RoomHandler/Handler)

	My MOTD library separates developer and administrator messages.  However, I
	find it handy to display them in one place.  This interface sends them to an
	output control.  Sending null will clear the control, but we'll want to send
	the messages at the same time to avoid clearing one while posting another.
	Here are some prefix and suffix procs to send simultaneously.
		bgf_GetAdminPrefix(Viewer)
		bgf_GetAdminSuffix(Viewer)

////////////////////////////////////////////////////////////////////////////////
bgf_RoomHandler
	parent_type		= /p_RoomHandler (Multiple Game Rooms library)

	ahandler.dm
		Change these names to overrule specific parts of my interface.
		Nullifying them will remove the behavior.
			bgf_GetControlLog(Person)
				Returns the output name "bgf_log".
			bgf_GetControlRoomsList(Person)
				Returns the grid name "bgf_roomsList".

		bgf_Log(Message)
			By default, this just outputs a Message to bgf_GetControlLog().
		bgf_UpdateRoomDisplay(bgf_Room/Room,Viewer)
			Rather than updating the entire list, this will update only Room.
			As with the full list displays, this will be ignored if
			bgf_controlRoomsList is null.


	chat.dm
		Change these names to overrule specific parts of my interface.
		Nullifying them will remove the behavior.
			bgf_GetControlChat()
				Returns the output name "bgf_chat".
			bgf_GetControlGlobalHear()
				Returns the checkable button name "bgf_globalHear".
			bgf_GetControlGlobalSpeak()
				Returns the checkable button name "bgf_globalSpeak".

		The following return verbs that get passed to players in the default
		interface.  For the default verb definitions, check chat.dm.
			bgf_GetVerbChat()
				Returns /bgf_RoomHandler/verb/bgf_chat.
			bgf_GetVerbGlobalHear()
				Returns /bgf_RoomHandler/verb/bgf_globalhear.
			bgf_GetVerbGlobalSpeak()
				Returns /bgf_RoomHandler/verb/bgf_globalspeak.

		bgf_Chat(Message="",Filter=TRUE,Sender,Prefix,Suffix,Chatters)
			Outputs Message to the bgf_controlChat of Chatters.
			If Chatters is not defined, it defaults to all people in all rooms.
			If Filter, Message will be filtered.  Prefix and Suffix will not.
			Sender is tracked for automated muting purposes.
			If Prefix and Suffix are null and not text, the prefix and suffix
			procs below will be called.
			Returns TRUE on success.
		bgf_GetChatPrefix(Person)
			Returns a combination of Person's avatar and name output by default.
			It's preceded by a span tag.
		bgf_GetChatSuffix(Person)
			Returns a closing span tag by default.

		The following help manage global chat settings.
			bgf_GetGlobalChatListeners()
				Returns th elist of all people who can hear global chat.
			bgf_IsHearingGlobal(Person)
				Returns whether Person can currently hear global messages.
			bgf_IsSpeakingGlobal(Person)
				Returns whether Person can currently send global messages.
			bgf_SetHearGlobal(Person,On=TRUE)
				Toggles whether Person hears global messages.
			bgf_SetSpeakGlobal(Person,On=TRUE)
				Toggles whether Person speaks global messages.


	create.dm
		Adjust this to change the control name or nullify it to remove the
		behavior.
			bgf_GetControlCreateRoom(Person)
				Returns the button name "bgf_createRoom".

		The following returns a verb that get passed to players in the default
		interface.  For the default verb definition, check create.dm.
			bgf_GetVerbCreateRoom()
				Returns /bgf_RoomHandler/verb/bgf_createroom.
		The verb calls this proc on failure.  By default, it just sends a message
		to Person through the chat control.  Spam is bad.  Feel free to do it
		better in your own interface.
			bgf_DisplayRoomCreationFailure(Person)

		bgf_AutomaticallyEnterCreatedRoom(Person)
			Returns whether or not Person will be sent to the room Person created.
		bgf_CanCreateRoom(Person)
			Returns whether or not Person can create a room.  TRUE by default.
*		bgf_GetNewRoomType(Request)
			This can be used to determine what type of room is added.
			By default, it returns /bgf_Room.


	host.dm
		Change these names to overrule specific parts of my interface.
		Nullifying them will remove the behavior.
			bgf_GetControlHost(Person)
				Returns the window name "bgf_options_server".
			bgf_GetControlOptions(Person)
				Returns the child name "bgf_childOptions".
			bgf_GetControlPhaseTime(Person)
				Returns the button name "bgf_phaseTime".
			bgf_GetControlReconnectTime(Person)
				Returns the button name "bgf_reconnectTime".

		The following return verbs that get passed to players in the default
		interface.  For the default verb definitions, check host.dm.
			bgf_GetVerbPhaseTime(Person)
				Returns /bgf_RoomHandler/verb/bgf_phasetime.
			bgf_GetVerbReconnectTime(Person)
				Returns /bgf_RoomHandler/verb/bgf_reconnecttime.


		A default spam filter is provided.  It can be overwritten.
			bgf_SpamFilter/bgf_spamFilter
				A spam filter for chat.
			bgf_GetSpamFilterType()
				Returns /bgf_SpamFilter by default.

		A default MOTD datum is provided.  It can also be overwritten.
			bgf_MOTD/bgf_motd
				Message Of The Day datum
			bgf_GetMOTDType()
				Returns /bgf_MOTD by default.

		bgf_phaseTime
			This is the time phases will end in after votes are passed in minutes.
			The default is 1 minute.  It is not saved.
		bgf_reconnectTime
			This is the time that players have to reconnect before being forced
			to quit in minutes.  The default is 3.  It is not saved.
		bgf_GetTicksPerMinute()
			Returns a rough idea of the number of ticks per minute.

		bgf_IsHost(Person)
			Returns whether or not Person is the host.
			By default, it just does a ckey() comparison with world.host.
		bgf_AddHostInterface(Person)
			Adds host controls such as those managing spam and the MOTD.
		bgf_RemoveHostInterface(Person)
			Removes host controls such as those managing spam and the MOTD.


	person.dm

		bgf_AssignAvatarURL(Person)
			Caches an avatar URL for future use.
			Person must be a mob or client.
			Returns TRUE on success.
		bgf_ClearAvatar(Person)
			Removes Person's avatar URL from the cache.
		bgf_GetAvatar(Person,Wait=FALSE, Class="bgf_avatar")
			Returns an HTML image of Person's BYOND Member avatar.
			Returns null on failure.
			If !Wait, null is returned while the URL is found and cached.
			The tag is given the class Class for CSS formatting.
		bgf_GetAvatarURL(Person,Wait=FALSE)
			Returns Person's avatar URL.  Person should be a mob or client.
			Returns null on failure.
			If !Wait, null is returned while the URL is found and cached.
		bgf_GetNameOutput(Person, Class="bgf_name")
			Returns Person's name formatted for an output control.
			Person should be the mob or client of a BYOND Member.
			By default, it turns names into referral links to their blogs.
		bgf_GetNewArrivalRoom(Person)
			Returns the room those just entering the server are placed in.
			By default, this is the first room listed.
			If there is no room, it tries to add a new one and return it.
		bgf_Person2Key(Person)
			Returns Person's key is Person is a mob or client.

////////////////////////////////////////////////////////////////////////////////
bgf_SpamFilter
	parent_type	= /sf_SpamFilter (SpamFilter library)
		That library includes Deadron's XML for saving purposes.  It's overkill
		and I apologize.  (I was used to creating cards with XML at the time.)
		I suggest using it elsewhere as well to make it worthwhile.

	The room handler is kept track of.
		var/bgf_RoomHandler/bgf_handler
		New(bgf_RoomHandler/Handler)

	Change these names to overrule specific parts of my interface.
	Nullifying them will remove the behavior.
		bgf_GetControlMaxLength()
			Returns the button name "bgf_Max_Length".
		bgf_GetControlPunishment()
			Returns the button name "bgf_Punishment".
		bgf_GetControlMaxChatsPerPeriod()
			Returns the button name "bgf_Max_Chats_Per_Period".
		bgf_GetControlPeriod()
			Returns the button name "bgf_Period".

////////////////////////////////////////////////////////////////////////////////
*/