bgf_RoomHandler

	//==========================================================================
	parent_type	= /p_RoomHandler

	//==========================================================================
	//--------------------------------------------------------------------------
	New()
		..()

		//	Create a spam filter and MOTD.

		var/myType				= src.bgf_GetSpamFilterType()
		if (myType)
			src.bgf_spamFilter	= new myType(src)
			src.bgf_spamFilter.sf_Load()

		myType				= src.bgf_GetMOTDType()
		if (myType)
			src.bgf_motd	= new myType(src)
			src.bgf_motd.motd_Load()

	//--------------------------------------------------------------------------
	//	Displays the collected rooms to a single mob or client.
	//	By default, this uses a grid with the following format:
	//		<JoinLink>	<RoomName>: <RoomStatus>
	//		<RoomPic>	<CommaSeparatedPlayers>
	//	If you don't want it, set bgf_controlRoomsList to null.
	p_DisplayRoomsToViewer(Viewer)
		var/myInterface		= src.bgf_GetControlRoomsList(Viewer)
		if (!myInterface)	return

		if ( ismob(Viewer) )
			var/mob/M					= Viewer
			Viewer						= M.client
		if ( !istype(Viewer,/client) )	return

		var/list/rooms	= src.p_GetRooms()
		winset(Viewer,myInterface,"cells=2x[2*length(rooms)]")

		var
			row		= 1
			bgf_Room/R
		for(R in rooms)
			Viewer	<< output(R.bgf_GetEnterLink(),"[myInterface]:1,[row]")
			Viewer	<< output(R.bgf_GetStatus(),"[myInterface]:2,[row]")
			row++
			Viewer	<< output(R.bgf_GetPicture(),"[myInterface]:1,[row]")
			Viewer	<< output(R.bgf_GetPlayersDisplay(),"[myInterface]:2,[row]")
			row++

	//--------------------------------------------------------------------------
	//	Displays the collected rooms to ALL viewers.
	//	By default, this uses a grid with the following format:
	//		<JoinLink>	<RoomName>: <RoomStatus>
	//		<RoomPic>	<CommaSeparatedPlayers>
	//	If you don't want it, set bgf_controlRoomsList to null.
	p_DisplayRoomsToAll(AutomatedCall=FALSE)
		if (AutomatedCall)	return
		var/myInterface		= src.bgf_GetControlRoomsList()
		if (!myInterface)	return

		//	Mass calls can be avoided by calling once in a batch.
		//if (AutomatedCall)	return

		var/list/rooms	= src.p_GetRooms()
		src.p_WinsetAll(myInterface,"cells=2x[2*length(rooms)]")

		var
			row		= 1
			bgf_Room/R
		for(R in rooms)
			src.p_OutputAll(R.bgf_GetEnterLink(),"[myInterface]:1,[row]")
			src.p_OutputAll(R.bgf_GetStatus(),"[myInterface]:2,[row]")
			row++
			src.p_OutputAll(R.bgf_GetPicture(),"[myInterface]:1,[row]")
			src.p_OutputAll(R.bgf_GetPlayersDisplay(),"[myInterface]:2,[row]")
			row++

	//--------------------------------------------------------------------------
	p_Move(Person,p_Room/Room,AutomatedCall=FALSE)
		if ( istype(Room) && Room.p_IsListed(Person) )	return FALSE
		return ..(Person,Room,AutomatedCall)

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Adjust these to overrule specific parts of my interface.
		bgf_GetControlLog(Person)
			return "bgf_log"//	output
		bgf_GetControlRoomsList(Person)
			return "bgf_roomsList"//	grid

		//----------------------------------------------------------------------
		//	By default, this just outputs Message to bgf_controlLog.
		bgf_Log(Message)
			var/myInterface		= src.bgf_GetControlLog()
			if (!myInterface)	return
			if ( !istext(Message) )
				src.p_OutputAll(Message,myInterface)
				return
			src.p_OutputAll("<span class='bgf_time'>\[[time2text(world.realtime,"hh:mm:ss")]\]</span> [Message]",myInterface)

		//----------------------------------------------------------------------
		//	Rather than updating the entire list, this will update only Room.
		//	As with the full list displays, this will be ignored if
		//	bgf_controlRoomsList is null.
		bgf_UpdateRoomDisplay(bgf_Room/Room,Viewer)
			if ( !istype(Room) )	return
			var/myInterface			= src.bgf_GetControlRoomsList(Viewer)
			if (!myInterface)		return

			var
				list/rooms	= src.p_GetRooms()
				index		= rooms.Find(Room)
			if (index<1)	return

			var/row			= 2*(index-1) + 1

			if (Viewer)
				Viewer	<< output(Room.bgf_GetEnterLink(),"[myInterface]:1,[row]")
				Viewer	<< output(Room.bgf_GetStatus(),"[myInterface]:2,[row]")
				row++
				Viewer	<< output(Room.bgf_GetPicture(),"[myInterface]:1,[row]")
				Viewer	<< output(Room.bgf_GetPlayersDisplay(),"[myInterface]:2,[row]")
			else
				src.p_OutputAll(Room.bgf_GetEnterLink(),"[myInterface]:1,[row]")
				src.p_OutputAll(Room.bgf_GetStatus(),"[myInterface]:2,[row]")
				row++
				src.p_OutputAll(Room.bgf_GetPicture(),"[myInterface]:1,[row]")
				src.p_OutputAll(Room.bgf_GetPlayersDisplay(),"[myInterface]:2,[row]")
