bgf_RoomHandler

	//==========================================================================
	var
		list/bgf_keys2avatars

	//==========================================================================
	//--------------------------------------------------------------------------
	//	Announce entrances.  Add possible host commands.
	//	New arrivals should be placed in a room.
	p_Reconnected(Person)
		if ( src.p_Find(Person) )
			..(Person)
		else
			//	Caching an avatar early in the hopes of it being displayed quickly.
			//	Spawning off because it takes time to contact the server.
			spawn(0)
				src.bgf_AssignAvatarURL(Person)
			src.p_Move(Person,src.bgf_GetNewArrivalRoom(Person),TRUE)
			src.p_AddInterface(Person)

		src.bgf_Log("<strong>[src.bgf_GetNameOutput(Person)]</strong> entered the server.")

	//--------------------------------------------------------------------------
	//	Announce exits and remove host commands.
	p_Disconnected(Person)
		src.bgf_Log("<strong>[src.bgf_GetNameOutput(Person)]</strong> exited the server.")
		src.p_RemoveInterface(Person)
		return ..(Person)

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Caches an avatar URL for future use.
		//	Person must be a mob or client.
		//	Returns TRUE on success.
		bgf_AssignAvatarURL(Person)
			var/client/C		= Person
			if ( ismob(Person) )
				var/mob/M		= Person
				C				= M.client
			if ( !istype(C) )	return FALSE

			if ( !C.IsByondMember() )	return FALSE

			var/list/serverData	= world.Export("http://www.byond.com/members/[ckey(C.key)]?format=text")
			if ( !serverData || !findtext(serverData["CONTENT-TYPE"],"text/plain") )
				return FALSE

			var/savefile/sf	= new
			sf.ImportText( "/", file2text(serverData["CONTENT"]) )
			sf.cd	= "general"

			if (!src.bgf_keys2avatars)	src.bgf_keys2avatars	= list()
			src.bgf_keys2avatars[C.key]	= sf["icon"]
			return TRUE

		//----------------------------------------------------------------------
		//	Removes Person's avatar URL from the cache.
		bgf_ClearAvatar(Person)
			if (src.bgf_keys2avatars)
				src.bgf_keys2avatars	-= src.bgf_Person2Key(Person)

		//----------------------------------------------------------------------
		//	Returns an HTML image of Person's BYOND Member avatar.
		//	Returns null on failure.
		//	If !Wait, null is returned while the URL is found and cached.
		//	The tag is given the class Class for CSS formatting.
		bgf_GetAvatar(Person,Wait=FALSE, Class="bgf_avatar")
			var/url		= src.bgf_GetAvatarURL(Person,Wait)
			if (!url)	return null
			return "<img class = '[Class]' src='[url]'>"

		//----------------------------------------------------------------------
		//	Returns Person's avatar URL.  Person should be a mob or client.
		//	Returns null on failure.
		//	If !Wait, null is returned while the URL is found and cached.
		bgf_GetAvatarURL(Person,Wait=FALSE)
			var/K	= src.bgf_Person2Key(Person)
			if (!K)	return null

			if (K in src.bgf_keys2avatars)	return src.bgf_keys2avatars[K]

			if ( !ismob(Person) && !istype(Person,/client) )	return null

			if (Wait)
				if ( src.bgf_AssignAvatarURL(Person) )
					return src.bgf_keys2avatars[K]
				return null

			//	Return null for now, but place the URL for the next call.
			.	= null
			spawn(0)
				src.bgf_AssignAvatarURL(Person)

		//----------------------------------------------------------------------
		//	Returns Person's name formatted for an output control.
		//	Person should be the mob or client of a BYOND Member.
		//	By default, it turns names into referral links to their blogs.
		bgf_GetNameOutput(Person, Class="bgf_name")
			.					= "<span class='[Class]'>[Person]</span>"

			var/client/C		= Person
			if ( ismob(Person) )
				var/mob/M		= Person
				C				= M.client
			if ( !istype(C) || !C.IsByondMember() )	return .

			return "<a class='[Class]' href='http://www.byond.com/members/[ckey(C.key)]?invite=[ckey(C.key)]'>[Person]</a>"

		//----------------------------------------------------------------------
		//	Returns the room those just entering the server are placed in.
		//	By default, this is the first room listed.
		//	If there is no room, it tries to add a new one and return it.
		bgf_GetNewArrivalRoom(Person)
			var/list/rooms			= src.p_GetRooms()
			if ( length(rooms) )	return rooms[1]

			var/rType				= src.bgf_GetNewRoomType(Person)
			if  ( ispath(rType) )
				var/R						= new rType(src)
				if ( src.p_AddRoom(R) )	return R

		//----------------------------------------------------------------------
		//	Returns Person's key is Person is a mob or client.
		bgf_Person2Key(Person)
			if ( ismob(Person) )
				var/mob/M	= Person
				return M.key
			if ( istype(Person,/client) )
				var/client/C	= Person
				return C.key
