/*
////////////////////////////////////////////////////////////////////////////////
bgf_Phase
	parent_type	= /ph_Phase (Phase library)

	bgf_CanUseInterface(Candidate)
		ph_Phase restricts interfaces to people the phase keeps track of by
		default.  I've redefined it to allow any person in the room.
		This will return Candidate's client if it can use an interface.

	Change this names to overrule specific parts of my interface.
	Nullifying it will remove the behavior.
		bgf_GetControlConfirmVote(Person)
			Returns the button name "bgf_confirmVote".

	The following return verbs that get passed to players in the default
	interface.  For the default verb definitions, check phase.dm.
		bgf_GetVerbConfirm(Person)
		bgf_GetVerbVote(Person)

*	bgf_GetNextPhaseType()
		This determines what phase comes next.  By default it returns null.

	bgf_GetRoom()
		Returns the room the phase takes place in.

	bgf_CanConfirm(Candidate)
		Returns whether or not Candidate can confirmthe end of their turn.
		By default, this is just whether the phase is tracking Candidate.
	bgf_Confirm(Candidate)
		As long as the above check passes, this removes the player from
		the phase to end its turn.

////////////////////////////////////////////////////////////////////////////////
bgf_PhaseEndingVotes
	parent_type	= /sidv_SharedIDVote (SharedIDVote library)

	The phase handler is tracked.
		var/bgf_PhaseHandler/bgf_phases
		New(bgf_PhaseHandler/Handler)

////////////////////////////////////////////////////////////////////////////////
bgf_PhaseHandler
	parent_type	= /ph_PhaseHandler (Phase library)

	The room is tracked.
		bgf_Room/bgf_room
		New(bgf_Room/Room)
		bgf_GetRoom()

	ph_GetNextPhase()
		If there is a current phase, this has been defined to ask it what comes
		next.

*	bgf_GetStartingPhaseType()
		Called when there is no phase to reference for the next.
		Returns /bgf_Phase by default.

	bgf_PhaseTimer/bgf_timer
	bgf_GetTimerType()
		Returns /bgf_PhaseTimer by default.

	bgf_PhaseEndingVotes/bgf_votes
	bgf_GetVotesType()
		Returns /bgf_PhaseEndingVotes by default.

////////////////////////////////////////////////////////////////////////////////
bgf_PhaseTimer
	parent_type	= /timer_Timer (Timer library)

	The phase handler is tracked.
		var/bgf_PhaseHandler/bgf_phases
		New(bgf_PhaseHandler/Handler)

	bgf_AnnounceStart(Time)
		By default, this sends a message to the chat output.  Spam is bad.
		You can do better in your own interface.

////////////////////////////////////////////////////////////////////////////////
bgf_Room
	parent_type	= /p_StartEndHandler (StartEnd library.  See below.)

	aroom.dm
		The room handler is tracked.
			bgf_RoomHandler/bgf_handler
			New(bgf_RoomHandler/Handler)
			bgf_GetHandler()

		bgf_DeleteIfEmpty()
			Returns whether or not the room is deleted when empty.
			Returns TRUE by default.
			Note that deletion will call the room's p_End() proc with itself as
			the argument.

		bgf_AllowReconnect(Player)
			Returns TRUE if Player is allowed to reconnect.  By default, this
			just makes sure Player is a mob with a key that is playing in the
			room.
		bgf_GetReconnectTicks()
			Returns the time disconnected players have to rejoin the server in
			ticks.  By default, this returns the handler's settings.
			Candidate is the disconnected player.

		bgf_PhaseHandler/bgf_phases
		bgf_GetPhaseHandlerType()
			The type of phase handler used can be defined or nullified.
			Returns /bgf_PhaseHandler by default.

		If the handler's bgf_controlRoomsList is defined, Topic() links are
		used to move people between rooms along with the following.
			bgf_GetEnterLink(Class="bgf_enter")
				Returns said Topic() link.
*			bgf_GetName()
				Returns the name of the room.  Redefine it.
*			bgf_GetPicture()
				Returns null by default.  Redefine it.
			bgf_GetPlayersDisplay(Class="bgf_players")
			bgf_GetStatus(Class="bgf_status")

		Redefine this to adjust my interface.
			bgf_GetControlChat(Person)		output

		bgf_Chat(Message="",Filter=TRUE,Sender,Prefix,Suffix,Chatters)
			Outputs Message to the bgf_controlChat of Chatters.
			If Chatters is not defined, it defaults to everyone in the room.
			If Filter, Message will be filtered.  Prefix and Suffix will not.
			Sender is tracked for automated muting purposes.
			If Prefix and Suffix are null and not text, the prefix and suffix
			procs below will be called.
			Returns TRUE on success.
		bgf_GetChatPrefix(Person)
			Returns a combination of Person's avatar and name output by default.
			It's preceded by a span tag.
		bgf_GetChatSuffix(Person)
			Returns a closing span tag by default.
		bgf_GetSpamFilter(Person)
			Return's the handler's spam filter by default.

		The following two procs just call the handler by default.  They can be
		redefined for individual rooms.
			bgf_GetAvatar(Person,Wait=FALSE, Class="bgf_avatar")
				Returns an HTML image of Person's BYOND Member avatar.
				Returns null on failure.
				If !Wait, null is returned while the URL is found and cached.
				The tag is given the class Class for CSS formatting.
			bgf_GetNameOutput(Person, Class="bgf_name")
				Returns Person's name formatted for an output control.
				Person should be the mob or client of a BYOND Member.
				By default, it turns names into referral links to their blogs.


	joinWaitStart.dm
		Change these names to overrule specific parts of my interface.
		Nullifying them will remove the behavior.
			bgf_GetControlJoinQuit(Person)
				Returns the button name "bgf_joinQuit".
			bgf_GetControlRoomies(Person)
				Returns the grid name "bgf_roomies"
			bgf_GetControlStart(Person)
				Returns the button name "bgf_startGame"

		The following return verbs that get passed to players in the default
		interface.  For the default verb definitions, check joinWaitStart.dm.
			bgf_GetVerbJoinQuit()
				Returns /bgf_Room/verb/bgf_joinquit.
			bgf_GetVerbStart()
				Returns /bgf_Room/verb/bgf_start.

		p_Ended(Ender,AutomatedCall=FALSE)
			Note that Ender will equal the room itself if it is being deleted.

		p_GetMinPlayers()
			By default, StartEndHandlers require a minimum of two players.
			We'll set the default to 1 to be friendly to single player games.

////////////////////////////////////////////////////////////////////////////////
p_StartEndHandler
	parent_type		= /p_Room (Multiple Game Rooms library)

	Normal parent_type	= /p_PeopleHandler (People library)

	This is a copy of my StartEnd extension to my People library.
	All I'm doing here is changing the parent_type to a p_Room.
	Unfortunately, BYOND won't overwrite the parent types of included libraries.

	http://www.byond.com/developer/ACWraith/StartEnd

////////////////////////////////////////////////////////////////////////////////
*/