bgf_Room
	parent_type	= /p_StartEndHandler

	//==========================================================================
	var
		bgf_RoomHandler/bgf_handler

		bgf_PhaseHandler/bgf_phases

	//==========================================================================
	//--------------------------------------------------------------------------
	New(bgf_RoomHandler/Handler)
		if ( istype(Handler) )	src.bgf_handler	= Handler
		var/myType	= src.bgf_GetPhaseHandlerType()
		if (myType)	src.bgf_phases	= new myType(src)
		..()

	//--------------------------------------------------------------------------
	Del()
		src.p_End(src,TRUE)
		if (src.bgf_handler)
			src.bgf_handler.p_RemoveRoom(src,TRUE)
			src.bgf_handler	= null
		..()

	//--------------------------------------------------------------------------
	//	If the handler's bgf_controlRoomsList is defined, topic links are used
	//	to move people between rooms.
	Topic(href,href_list[])
		var/bgf_RoomHandler/H	= src.bgf_GetHandler()
		if ( !istype(H) || !H.bgf_GetControlRoomsList(usr) )
			return ..(href,href_list)

		//	Only spectators can join rooms.
		//	Ignore players.  Transform waiters.
		if ( src.p_IsPlaying(usr) )			return ..(href,href_list)
		if ( src.p_IsWaiting(usr) )
			if ( src.p_IsStarting() || src.p_IsEnding() )
				return ..(href,href_list)
			if ( !src.p_AddSpectator(usr) )	return ..(href,href_list)

		src.bgf_handler.p_Move(usr,src,FALSE)

		return ..(href,href_list)

	//==========================================================================
	//--------------------------------------------------------------------------
	//	Automatically remove disconnected mobs.
	//	However, give those actually playing a chance to reconnect.
	p_Disconnected(Person)
		.	= ..(Person)
		if ( src.bgf_AllowReconnect(Person) && ismob(Person) )
			var
				mob/M	= Person
				myTicks	= src.bgf_GetReconnectTicks(Person)
			if ( isnum(myTicks) && (myTicks>=0) )
				spawn( src.bgf_GetReconnectTicks(Person) )
					if (!M || !M.client)	src.p_Exited(M)

	//--------------------------------------------------------------------------
	p_Entered(Person,p_Room/OldRoom,AutomatedCall=FALSE)
		.	= ..(Person,AutomatedCall)
		if (src.bgf_handler)	src.bgf_handler.bgf_UpdateRoomDisplay(src)

	//--------------------------------------------------------------------------
	p_Exited(Person,AutomatedCall=FALSE)
		.	= ..(Person,AutomatedCall)

		if (src.bgf_handler)
			//	During transitions, Person won't be inside a room.
			//	That means updates will have to be sent directly.

			//	Rooms with no people are deleted.
			if ( src.bgf_DeleteIfEmpty() && !length(src.p_GetListOfAll()) )
				src.bgf_handler.p_RemoveRoom(src,TRUE)
				src.bgf_handler.p_DisplayRoomsToViewer(Person)
				del src

			if (!AutomatedCall)
				src.bgf_handler.bgf_UpdateRoomDisplay(src)
				src.bgf_handler.bgf_UpdateRoomDisplay(src,Person)

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	The handler var is handled internally.
		//	Redefining this proc is not advised.
		bgf_GetHandler()
			return src.bgf_handler


		//----------------------------------------------------------------------
		//	Returns whether or not the room is deleted when empty.
		//	Returns TRUE by default.
		bgf_DeleteIfEmpty()
			return TRUE

		//----------------------------------------------------------------------
		//	Returns TRUE if Player is allowed to reconnect.
		//	By default, this just makes sure Player is a mob with a key that
		//	is playing in the room.
		bgf_AllowReconnect(Player)
			if ( !ismob(Player) || !src.p_IsPlaying(Player) )	return FALSE
			var/mob/M	= Player
			if (!M.key)	return FALSE
			return TRUE
		//----------------------------------------------------------------------
		//	Returns the time disconnected players have to rejoin the server in
		//	ticks.  By default, this returns the handler's settings.
		//	Candidate is the disconnected player.
		bgf_GetReconnectTicks(Candidate)
			if (!src.bgf_handler)	return 0
			.	= src.bgf_handler.bgf_reconnectTime
			.	*= src.bgf_handler.bgf_GetTicksPerMinute()

		//----------------------------------------------------------------------
		//	I find it handy to name rooms.
		bgf_GetName()
			return "Untitled Room"


		//	These are used by the handler when the room is displayed in its list.
		//----------------------------------------------------------------------
		bgf_GetEnterLink(Class="bgf_enter")
			return "<a class='[Class]' href='?src=\ref[src];'>Enter [src.bgf_GetName()]</a>"
		//----------------------------------------------------------------------
		bgf_GetPicture()
			return null
		//----------------------------------------------------------------------
		bgf_GetPlayersDisplay(Class="bgf_players")
			.	= ""
			for(var/P in src.p_GetListOfAll())	.	+= "[P], "
			if (.)
				.	= copytext(.,1,length(.)-1)
				.	= "<span class='[Class]'>[.]</span>"
		//----------------------------------------------------------------------
		bgf_GetStatus(Class="bgf_status")
			if ( (src.p_IsInProgress() || src.p_IsStarting()) && !src.p_IsEnding() )
				.	= "In progress."
			else
				.	= "Awaiting players."
			.	= "<span class='[Class]'>[.]</span>"


		//----------------------------------------------------------------------
		//	Outputs Message to the bgf_controlChat of Chatters.
		//	If Chatters is not defined, it defaults to everyone in the room.
		//	If Filter, Message will be filtered.  Prefix and Suffix will not.
		//	Sender is tracked for automated muting purposes.
		//	If Prefix and Suffix are null and not text, the prefix and suffix
		//	procs below will be called.
		//	Returns TRUE on success.
		bgf_Chat(Message="",Filter=TRUE,Sender,Prefix,Suffix,Chatters)
			if (!src.bgf_handler)	return FALSE
			var/myInterface			= src.bgf_GetControlChat(Sender)
			if (!myInterface)		return FALSE

			if (Filter)
				var/bgf_SpamFilter/SF	= src.bgf_GetSpamFilter(Sender)
				if ( istype(SF) )		Message	= SF.sf_Filter(Sender,Message)

			if (!Message)	return TRUE

			if (!Prefix && !istext(Prefix))	Prefix	= src.bgf_GetChatPrefix(Sender)
			if (!Suffix && !istext(Suffix))	Suffix	= src.bgf_GetChatSuffix(Sender)
			if (!Chatters)					Chatters	= src.bgf_handler.p_GetPeople()

			Chatters		<< output("[Prefix][Message][Suffix]",myInterface)
			return TRUE
		//----------------------------------------------------------------------
		bgf_GetChatPrefix(Person)
			if (src.bgf_handler)
				return "[src.bgf_GetAvatar(Person)] [src.bgf_GetNameOutput(Person)]: "
			return "[Person]"
		//----------------------------------------------------------------------
		bgf_GetChatSuffix(Person)
		//----------------------------------------------------------------------
		bgf_GetControlChat(Person)
			var/bgf_RoomHandler/H	= src.bgf_GetHandler()
			if ( istype(H) )		return H.bgf_GetControlChat(Person)
		//----------------------------------------------------------------------
		bgf_GetSpamFilter(Person)
			var/bgf_RoomHandler/H	= src.bgf_GetHandler()
			if ( istype(H) )		return H.bgf_spamFilter


		//----------------------------------------------------------------------
		//	Returns an HTML image of Person's BYOND Member avatar.
		//	Returns null on failure.
		//	If !Wait, null is returned while the URL is found and cached.
		//	The tag is given the class Class for CSS formatting.
		bgf_GetAvatar(Person,Wait=FALSE, Class="bgf_avatar")
			var/bgf_RoomHandler/H	= src.bgf_GetHandler()
			if ( istype(H) )	return H.bgf_GetAvatar(Person,Wait,Class)
		//----------------------------------------------------------------------
		//	Returns Person's name formatted for an output control.
		//	Person should be the mob or client of a BYOND Member.
		//	By default, it turns names into referral links to their blogs.
		bgf_GetNameOutput(Person, Class="bgf_name")
			var/bgf_RoomHandler/H	= src.bgf_GetHandler()
			if ( istype(H) )	return H.bgf_GetNameOutput(Person,Class)


		//----------------------------------------------------------------------
		//	The type of phase handler used can be defined or nullified.
		bgf_GetPhaseHandlerType()
			return /bgf_PhaseHandler
