////////////////////////////////////////////////////////////////////////////////
//	This is a copy of my StartEnd extension to my People library.
//	All I'm doing here is changing the parent_type to a p_Room.
//	In an ideal world, this could be accomplished with the first 3 lines.
//	Unfortunately, BYOND won't overwrite the parent types of included libraries.
////////////////////////////////////////////////////////////////////////////////

p_StartEndHandler
	//parent_type	= /p_PeopleHandler
	parent_type		= /p_Room

	//==========================================================================
	var
		p_isStarting	= FALSE
		p_isInProgress	= FALSE
		p_isEnding		= FALSE

	//==========================================================================
	//--------------------------------------------------------------------------
	p_AddWaiter(Waiter,AutomatedCall=FALSE)
		//	Only allow a certain amount of waiters?
		var
			maxW							= src.p_GetMaxWaiters()
			list/waiters
		if (maxW >= 0)
			waiters							= src.p_GetListOfWaiters()
			if (length(waiters) >= maxW)	return FALSE

		//	Call PeopleHandler
		.			= ..(Waiter,AutomatedCall)
		if ( !(.) )	return .

		//	Automatically start the game?
		if ( !src.p_StartWhenMaxPlayersWaiting() )	return .
		//	No need.  There are not enough people waiting.
		//	(Or players are not limited.)
		var/maxP	= src.p_GetMaxPlayers()
		if ( (maxP < 0) || (length(src.p_GetListOfWaiters()) < maxP) )
			return .

		src.p_Start(Waiter,TRUE)

	//--------------------------------------------------------------------------
	p_RemovePlayer(Player,AutomatedCall=FALSE)
		.	= ..(Player,AutomatedCall)
		if ( . && src.p_EndWhenLessThanMinPlayers() )
			var/minP	= src.p_GetMinPlayers()
			if ( (minP >= 0) && (length(src.p_GetListOfPlayers()) < minP) )
				src.p_End(null,AutomatedCall)

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns the maximum number of players.
		//	If this is negative, the limit is ignored.
		//	By default, this returns -1.
		p_GetMaxPlayers()
			return -1

		//----------------------------------------------------------------------
		//	Returns the minimum number of players.
		//	If this is negative, the limit is ignored.
		//	By default, this returns 2.
		p_GetMinPlayers()
			return 2

		//----------------------------------------------------------------------
		//	Returns the maximum number of waiters.
		//	If this is negative, the limit is ignored.
		//	By default, this returns -1.
		p_GetMaxWaiters()
			return -1

		//----------------------------------------------------------------------
		//	The game has ended.
		//	If the flag procs are set to start when there are enough waiters to
		//	meet the player limit, then p_Start() might be called automatically.
		p_End(Ender,AutomatedCall=FALSE)
			//	Already busy?
			if (src.p_IsStarting() || !src.p_IsInProgress() || src.p_IsEnding())
				return FALSE
			.	= TRUE

			//	We're ending.
			src.p_isEnding		= TRUE

			//	End.
			src.p_Ended(Ender,AutomatedCall)

			//	src is no longer in progress.
			src.p_isInProgress	= FALSE

			//	Call proc flag before the end flag is turned off.
			//	That way, the proc can be flag-dependant.
			var/checkAutoStart	= src.p_StartWhenMaxPlayersWaiting()

			//	We're done ending.
			src.p_isEnding		= FALSE

			//	Don't start again?
			if (!checkAutoStart)	return .
			//	No need.  There are not enough people waiting.
			//	(Or players are not limited.)
			var/maxP	= src.p_GetMaxPlayers()
			if ( (maxP < 0) || (length(src.p_GetListOfWaiters()) < maxP) )
				return .

			spawn(0)
				src.p_Start(null,TRUE)

		//----------------------------------------------------------------------
		//	Define this to handle the end of your game.
		//	By default, this simply changes all players into spectators.
		p_Ended(Ender,AutomatedCall=FALSE)
			//	Change all players into spectators.
			for(var/player in src.p_GetListOfPlayers())
				src.p_AddSpectator(player,TRUE)

		//----------------------------------------------------------------------
		//	This determines whether or not src automatically ends when
		//	a removed player drops the count below the minimum.
		//	By default, this returns TRUE if the minimum is a positive number.
		p_EndWhenLessThanMinPlayers()
			return (src.p_GetMinPlayers() > 0)

		//----------------------------------------------------------------------
		//	Returns whether or not src is in the middle of ending.
		p_IsEnding()
			return src.p_isEnding

		//----------------------------------------------------------------------
		//	Returns whether the game has been started and not ended.
		p_IsInProgress()
			return src.p_isInProgress

		//----------------------------------------------------------------------
		//	Returns whether or not src is in the middle of starting.
		p_IsStarting()
			return src.p_isStarting

		//----------------------------------------------------------------------
		//	The game has started.
		//	By default, this changes those in the waiting list into players.
		//	If there is a maximum player limit, some might be left waiting.
		//	If there are excess waiters, then they will be turned into spectators.
		//	Returns TRUE on success.
		p_Start(Starter,AutomatedCall=FALSE)
			//	Already busy?
			if (src.p_IsStarting() || src.p_IsInProgress() || src.p_IsEnding())
				return FALSE

			//	Not enough players?
			var
				list/waiters	= src.p_GetListOfWaiters()
				numP			= src.p_GetMinPlayers()
			if ( (numP > 0) && (numP > length(waiters)) )	return FALSE

			//	We're starting!
			.					= TRUE
			src.p_isStarting	= TRUE

			//	Only take enough waiters to meet the player limit.
			numP										= src.p_GetMaxPlayers()
			if ( (numP >= 0) && (numP < waiters.len) )	waiters.Cut(numP+1,0)

			//	Change all in waiters into players.
			var/waiter
			for(waiter in waiters)	src.p_AddPlayer(waiter,TRUE)

			//	Turn excess waiters into spectators.
			numP						= src.p_GetMaxWaiters()
			if (numP >= 0)
				waiters					= src.p_GetListOfWaiters()
				waiters					= waiters.Copy(numP+1,0)
				for(waiter in waiters)	src.p_AddSpectator(waiter,TRUE)

			//	We've begun!
			src.p_Started(Starter,AutomatedCall)

			//	The start process is done and src is in progress.
			src.p_isInProgress	= TRUE
			src.p_isStarting	= FALSE

		//----------------------------------------------------------------------
		//	Define this to handle the beginning of your game.
		//	By default, this does nothing.
		p_Started(Starter,AutomatedCall=FALSE)

		//----------------------------------------------------------------------
		//	This determines whether or not src automatically begins when
		//	there are enough people waiting to meet the player limit.
		//	By default, this returns TRUE if the maximum player limit is a
		//	positive number.
		p_StartWhenMaxPlayersWaiting()
			return (src.p_GetMaxPlayers() > 0)
